#ifndef __FAKE_ARMV7_ELF_H
#define __FAKE_ARMV7_ELF_H

#include <asm/ptrace.h>
#ifndef PSR_A_BIT
#include <asm/user.h>
#else
#include <sys/user.h>
#endif
#ifdef __KERNEL
#include <asm/procinfo.h>
#endif

typedef unsigned long elf_greg_t;
typedef unsigned long elf_freg_t[3];

#define EM_ARM 40
#define EF_ARM_APCS26 0x08
#define EF_ARM_SOFT_FLOAT 0x200
#define EF_ARM_EABI_MASK 0xFF000000

#define R_ARM_NONE 0
#define R_ARM_PC24 1
#define R_ARM_ABS32 2
#define R_ARM_CALL 28
#define R_ARM_JUMP24 29

#define ELF_NGREG (sizeof (struct pt_regs) / sizeof(elf_greg_t))
typedef elf_greg_t elf_gregset_t[ELF_NGREG];

#ifdef PSR_A_BIT
/* Define user_fp structure for Android platform-21 and above */
struct user_fp {
 struct __fp_reg {
 unsigned int sign1:1;
 unsigned int unused:15;
 unsigned int sign2:1;
 unsigned int exponent:14;
 unsigned int j:1;
 unsigned int mantissa1:31;
 unsigned int mantissa0:32;
 } fpregs[8];
 unsigned int fpsr:32;
 unsigned int fpcr:32;
 unsigned char ftype[8];
 unsigned int init_flag;
};

#endif

typedef struct user_fp elf_fpregset_t;

#define elf_check_arch(x) ( ((x)->e_machine == EM_ARM) && (ELF_PROC_OK((x))) )

#define ELF_CLASS ELFCLASS32
#ifdef __ARMEB__
#define ELF_DATA ELFDATA2MSB
#else
#define ELF_DATA ELFDATA2LSB
#endif
#define ELF_ARCH EM_ARM

#define USE_ELF_CORE_DUMP
#define ELF_EXEC_PAGESIZE 4096

#define ELF_ET_DYN_BASE (2 * TASK_SIZE / 3)

#define ELF_PLAT_INIT(_r, load_addr) (_r)->ARM_r0 = 0

#define ELF_HWCAP (elf_hwcap)

#define ELF_PLATFORM_SIZE 8

#define ELF_PLATFORM (elf_platform)

#endif
